/*******************************************************************************

Copyright (c) 1997-2004, Perforce Software, Inc.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1.  Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

2.  Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTR
IBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL PERFORCE SOFTWARE, INC. BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

/*******************************************************************************
 * Name		: clientuserruby.h
 *
 * Author	: Tony Smith <tony@perforce.com> or <tony@smee.org>
 *
 * Description	: Ruby bindings for the Perforce API. User interface class
 * 		  for getting Perforce results into Ruby.
 *
 ******************************************************************************/

/*******************************************************************************
 * ClientUserRuby - the user interface part. Gets responses from the Perforce
 * server, and converts the data to Ruby form for returning to the caller.
 ******************************************************************************/
class ClientUserRuby : public ClientUser
{
    public:
	ClientUserRuby();

	// Client User methods overridden here
	void	HandleError( Error *e );
	void    OutputText( const_char *data, int length );
	void    OutputInfo( char level, const_char *data );
	void	OutputStat( StrDict *values );
	void    OutputBinary( const_char *data, int length );
	void	InputData( StrBuf *strbuf, Error *e );
	void	Diff( FileSys *f1, FileSys *f2, int doPage, 
				char *diffFlags, Error *e );

	void	Finished();

	// Local methods
	VALUE	 SetInput( VALUE i );
	P4Result& GetResults()		{ return results;	} 
	int	 ErrorCount();
	void	 Reset();

	// GC support
	void	GCMark();

	// Debugging support
	void	SetDebug( int d )	{ debug = d; 		}

    private:
	int	HashToForm( VALUE i, StrBuf *strbuf );
	VALUE	DictToHash( StrDict *form, StrPtr *specDef );
	void	SplitKey( const StrPtr *key, StrBuf &base, StrBuf &index );
	void	InsertItem( VALUE hash, const StrPtr *var, const StrPtr *val );

    private:
	P4Result	results;
	VALUE		input;
	int		debug;
};

