/*******************************************************************************

Copyright (c) 1997-2004, Perforce Software, Inc.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1.  Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

2.  Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTR
IBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL PERFORCE SOFTWARE, INC. BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/


/*******************************************************************************
 * Name		: p4result.cc
 *
 * Author	: Tony Smith <tony@perforce.com> or <tony@smee.org>
 *
 * Description	: Ruby class for holding results of Perforce commands 
 *
 ******************************************************************************/

#include <ruby.h>
#include <clientapi.h>
#include "gc_hack.h"
#include "p4result.h"

P4Result::P4Result()
{
    output = rb_ary_new();
    warnings = rb_ary_new();
    errors = rb_ary_new();
}


void
P4Result::Reset()
{
    output = rb_ary_new();
    warnings = rb_ary_new();
    errors = rb_ary_new();
}

void
P4Result::AddOutput( const char *msg )
{
    rb_ary_push( output, rb_str_new2( msg ) );
}

void
P4Result::AddOutput( VALUE out )
{
    rb_ary_push( output, out );
}

void
P4Result::AddError( Error *e )
{
    StrBuf	m;
    e->Fmt( &m );

    int s;
    s = e->GetSeverity();

    // 
    // Empty and informational messages are pushed out as output as nothing
    // worthy of error handling has occurred. Warnings go into the warnings
    // list and the rest are lumped together as errors.
    //

    if ( s == E_EMPTY || s == E_INFO )
	AddOutput( m.Text() );
    else if ( s == E_WARN )
	rb_ary_push( warnings, rb_str_new2( m.Text() ) );
    else
	rb_ary_push( errors, rb_str_new2( m.Text() ) );
}

int
P4Result::ErrorCount()
{
    return Length( errors );
}

int
P4Result::WarningCount()
{
    return Length( warnings );
}


int
P4Result::Length( VALUE ary )
{
    ID		iLength;
    VALUE 	len;
    int		count = 0;

    iLength = rb_intern( "length" );

    len = rb_funcall( ary, iLength, 0 );
    return NUM2INT( len );
}

void
P4Result::GCMark()
{
    rb_gc_mark( output );
    rb_gc_mark( errors );
    rb_gc_mark( warnings );
}
